module WsMessage exposing (Action(..), Publisher, WsMessage, createSession, decode, msgDecoder, strToAction)

import Dict exposing (Dict)
import Json.Decode exposing (Decoder, andThen, decodeString, fail, field, map2, map3, string, succeed)
import Json.Encode as Enc
import Model exposing (Model)


type Action
    = Publish
    | CreateSession


stringToAction : Dict String Action
stringToAction =
    Dict.fromList
        [ ( "Publish", Publish )
        , ( "CreateSession", CreateSession )
        ]


actionToString : Action -> String
actionToString action =
    Dict.filter
        (\_ v -> v == action)
        stringToAction
        |> Dict.keys
        |> List.head
        |> Maybe.withDefault ""


strToAction : String -> Decoder Action
strToAction str =
    case Dict.get str stringToAction of
        Just val ->
            succeed val

        Nothing ->
            fail ("invalid action '" ++ str ++ "'")


type alias Publisher =
    { uuid : String
    , name : String
    }


type alias WsMessage =
    { action : Action
    , publisher : Publisher
    , payload : String
    }



-- DECODERS


msgDecoder : Decoder WsMessage
msgDecoder =
    map3 WsMessage
        (field "action" string |> andThen strToAction)
        (field "publisher"
            (map2 Publisher
                (field "uuid" string)
                (field "name" string)
            )
        )
        (field "payload" string)


decode : String -> Maybe WsMessage
decode str =
    case decodeString msgDecoder str of
        Ok msg ->
            Just msg

        Err _ ->
            Nothing



-- ENCODERS


createSession : Model -> String
createSession model =
    Enc.object
        [ ( "action", Enc.string (actionToString CreateSession) )
        , ( "publisher"
          , Enc.object
                [ ( "uuid", Enc.string model.userUuid )
                , ( "name", Enc.string model.userName )
                ]
          )
        , ( "payload", Enc.string "" )
        ]
        |> Enc.encode 0
